/*************************************************************************
* BROWSE.HXX
* Wed Oct 26 13:08:16 1994
* (c) 1992-1994 STAR DIVISION
*************************************************************************/
#include "limits.h"
#ifndef _SV_HXX
#include "sv.hxx"
#endif

#ifndef _SVX_BRWBOX_HXX
#define _SVX_BRWBOX_HXX

#ifndef _SV_HXX
#endif

#define BROWSER_INVALIDID			USHRT_MAX
#define BROWSER_ENDOFSELECTION		-1

typedef int BrowserMode;
#define BROWSER_COLUMNSELECTION           1
#define BROWSER_MULTISELECTION            2
#define BROWSER_THUMBDRAGGING             4
#define BROWSER_KEEPHIGHLIGHT             8
#define BROWSER_KEEPSELECTION           BROWSER_KEEPHIGHLIGHT
#define BROWSER_HLINESFULL               16
#define BROWSER_VLINESFULL               32
#define BROWSER_HLINESDOTS               64
#define BROWSER_VLINESDOTS              128

#define BROWSER_NONE					  0
#define BROWSER_SELECT					720
#define BROWSER_EXPANDSELECTION 		721
#define BROWSER_ENHANCESELECTION		722
#define BROWSER_SELECTALL				723
#define BROWSER_SELECTDOWN				724
#define BROWSER_SELECTUP				725
#define BROWSER_CURSORDOWN				731
#define BROWSER_CURSORUP				732
#define BROWSER_CURSORLEFT				733
#define BROWSER_CURSORRIGHT 			734
#define BROWSER_CURSORPAGEDOWN			735
#define BROWSER_CURSORPAGEUP			736
#define BROWSER_CURSORPAGERIGHT 		735
#define BROWSER_CURSORPAGELEFT			736
#define BROWSER_CURSORENDOFFILE 		741
#define BROWSER_CURSORTOPOFFILE 		742
#define BROWSER_CURSORENDOFSCREEN		743
#define BROWSER_CURSORTOPOFSCREEN		744
#define BROWSER_CURSORHOME				745
#define BROWSER_CURSOREND				746
#define BROWSER_SCROLLDOWN				751
#define BROWSER_SCROLLUP				752

class BrowserColumn;
class BrowserColumns;
class BrowserDataWin;
class MultiSelection;

class BrowseEvent
{
    Window*     pWin;
    long        nRow;
    USHORT      nCol;
    USHORT      nColId;
    Rectangle   aRect;

public:
    BrowseEvent();
	BrowseEvent( Window* pWindow,
                 long nAbsRow, USHORT nColumn, USHORT nColumnId,
                 const Rectangle& rRect );

    Window*     GetWindow() const { return pWin; }
    long        GetRow() const { return nRow; }
    USHORT      GetColumn() const { return nCol; }
    USHORT      GetColumnId() const { return nColId; }
    Rectangle   GetRect() const { return aRect; }
};

class BrowserMouseEvent: public MouseEvent, public BrowseEvent
{
public:
	BrowserMouseEvent();
	BrowserMouseEvent( BrowserDataWin* pWin, const MouseEvent& rEvt );
	BrowserMouseEvent( Window* pWin, const MouseEvent& rEvt,
					   long nAbsRow, USHORT nColumn, USHORT nColumnId,
					   const Rectangle& rRect );
};

class BrowserDropEvent: public DropEvent, public BrowseEvent
{
public:
	BrowserDropEvent();
	BrowserDropEvent( BrowserDataWin* pWin, const DropEvent& rEvt );
};

class LScrollBar;

class BrowseBox: public Control
{
friend class BrowserDataWin;
friend class BrowserColumn;
friend class StringColumn;
friend class BitmapColumn;
friend class HandleColumn;

    Window         *pDataWin;       
	LScrollBar	   *pVScroll;		
	ScrollBar		aHScroll;		

    short           nDataRowHeight; 
    USHORT          nTitleLines;    
	USHORT			nControlAreaWidth; 
	BOOL			bThumbDragging; 
	BOOL			bColumnCursor;	
	BOOL			bMultiSelection;
	BOOL			bKeepHighlight; 
	BOOL			bHLines;		
	BOOL			bVLines;		
	BOOL			bHDots; 		
	BOOL			bVDots; 		

	BOOL			bBootstrapped;	
	BOOL			bHasBitmapHandle; 
    USHORT          nFirstCol;      
    long            nTopRow;        
    long            nCurRow;        
    USHORT          nCurColId;      

    BOOL            bSelecting;     
	BOOL			bDrag;
	BOOL			bHit;
	BOOL			bRubber;
    Point           a1stPoint, a2ndPoint;

	BOOL			bResizing;		
    USHORT          nResizeCol;     
    short           nResizeX;       
    short           nMinResizeX;    
    short           nDragX;         

    BOOL            bSelect;        
	BOOL			bSelectionIsVisible; 
    BOOL            bScrolling;     
    BOOL            bNotToggleSel;  
    BOOL            bHasFocus;      
	unsigned		nCursorHidden;	
    LRange          aSelRange;      

	BrowserColumns* pCols;			
	union
	{
		MultiSelection* pSel;		
		long			nSel;		
	}				uRow;
	MultiSelection* pColSel;		

private:
	void			Construct(BrowserMode nMode);
	void			Bootstrap();
    void            ExpandRowSelection( const BrowserMouseEvent& rEvt );
	void			ToggleSelection( BOOL bForce = FALSE );
	void			DrawCursor();
	void			ShowCursor()
					{ if ( --nCursorHidden == 0 ) DrawCursor(); }
	void			HideCursor()
					{ if ( nCursorHidden == 0 ) DrawCursor(); ++nCursorHidden; }

    virtual void    Resize();
    virtual void    Paint( const Rectangle& rRect );
	void			PaintData( Window& rWin, const Rectangle& rRect );

	void			UpdateScrollbars();

    short           ImpGetDataRowHeight() const;
    Rectangle       ImplFieldRectPixel( long nRow, USHORT nColId ) const;
	USHORT			ColCount() const;
    USHORT          FrozenColCount() const;
    void            ColumnInserted( USHORT nPos );
    long            ScrollHdl( ScrollBar* pBar );
    long            EndScrollHdl( ScrollBar* pBar );

	short			GetFrozenWidth() const;

protected:
    virtual void    MouseButtonDown( const MouseEvent& rEvt );
    virtual void    MouseMove( const MouseEvent& rEvt );
    virtual void    MouseButtonUp( const MouseEvent& rEvt );
    virtual void    KeyInput( const KeyEvent& rEvt );
	virtual void	LoseFocus();
	virtual void	GetFocus();

    virtual long    GetRowCount() const = 0;
    virtual BOOL    SeekRow( long nRow ) = 0;
	virtual void	PaintField( OutputDevice& rDev, const Rectangle& rRect,
								USHORT nColumnId ) const = 0;

	virtual void	MouseButtonDown( const BrowserMouseEvent& rEvt );
	virtual void	MouseMove( const BrowserMouseEvent& rEvt );
	virtual void	MouseButtonUp( const BrowserMouseEvent& rEvt );
    virtual void    StartScroll();
    virtual void    EndScroll();

	
    virtual void    Select();
    virtual void    DoubleClick();
	virtual void	CursorMoved();
    virtual void    ColumnResized( USHORT nColId );

	
	virtual BOOL			QueryDrop( const BrowserDropEvent& rEvt );
	virtual BOOL			Drop( const BrowserDropEvent& rEvt );
	virtual BOOL			StartDragging( Pointer& rMovePtr,
										   Pointer& rCopyPtr );
	virtual void			EndDragging( const DropAction &rAction );

public:
					BrowseBox( Window* pParent, WinBits nBits = 0,
							   BrowserMode nMode = 0 );
                    BrowseBox( Window* pParent, const ResId& rId,
							   BrowserMode nMode = 0 );
					~BrowseBox();

	
	BOOL			ChangeUpdateMode( BOOL bUpdate );
	void			Invalidate();
	void			Invalidate( const Rectangle& rRect )
                    { Control::Invalidate( rRect ); }
    void            EnableDrop( BOOL bDrop = TRUE )
					{ pDataWin->EnableDrop( bDrop ); }
	BOOL			IsDropEnabled() const
					{ return pDataWin->IsDropEnabled(); }

    MapMode         ChangeMapMode( const MapMode& rNewMapMode );
    Font            ChangeFont( const Font& rNewFont );
    Font            GetFont() const { return pDataWin->GetFont(); }
    Font            ChangeTitleFont( const Font& rNewFont );
    Font            GetTitleFont() const { return Control::GetFont(); }

	void			InsertHandleColumn( USHORT nWidth, BOOL bBitmap = FALSE );
	void			InsertDataColumn(
                        USHORT nColumnId, const String &rTitle,
                        USHORT nWidth, USHORT nPos = USHRT_MAX );
    void            SetColumnTitle( USHORT nColumnId, const String &rTitle );
    void            SetColumnWidth( USHORT nColumnId, USHORT nWidth );
    void            SetColumnPos( USHORT nColumnId, USHORT nPos );
    void            FreezeColumn( USHORT nColumnId, BOOL bFreeze = TRUE );
    void            UnfreezeColumns();
    void            RemoveColumn( USHORT nColumnId );
    void            RemoveColumns();

	void			SetDataRowHeight( USHORT nPixel );
    short           GetDataRowHeight() const
					{ return nDataRowHeight ? nDataRowHeight : ImpGetDataRowHeight(); }
    void            SetTitleLines( USHORT nLines );
    USHORT          GetTitleLines() const { return nTitleLines; }
    short           GetTitleHeight() const
					{ Size aSz( GetTextSize("") );
					  return nTitleLines ? nTitleLines * aSz.Height() + 4 : 0; }

    String          GetColumnTitle( USHORT nColumnId ) const;
    Rectangle       GetFieldRect( USHORT nColumnId ) const;
    USHORT          GetColumnWidth( USHORT nColumnId ) const;
    USHORT          GetColumnId( USHORT nPos ) const;
    USHORT          GetColumnPos( USHORT nColumnId ) const;
    BOOL            IsFrozen( USHORT nColumnId ) const;

    void            ToTop();
    void            ToBottom();
    void            ResetScroll();
    short           ScrollColumns( short nColumns );
	long			ScrollRows( long nRows );
	long			ScrollPages( long nPagesY );

    long            GetCurRow() const { return nCurRow; }
    USHORT          GetCurColumnId() const { return nCurColId; }
    BOOL            GoToRow( long nRow );
	BOOL			GoToColumnId( USHORT nColId );


    void            SetNoSelection();
    void            SelectAll();
	void			SelectRow( long nRow, BOOL bSelect = TRUE, BOOL bExpand = TRUE );
	void			SelectColumnPos( USHORT nCol, BOOL bSelect = TRUE );
	void			SelectColumnId( USHORT nColId, BOOL bSelect = TRUE )
					{ SelectColumnPos( GetColumnPos(nColId), bSelect ); }
	long			GetSelectRowCount() const;
    USHORT          GetSelectColumnCount() const;
    BOOL            IsRowSelected( long nRow ) const;
    BOOL            IsColumnSelected( USHORT nColumnId ) const;
	long			FirstSelectedRow( BOOL bInverse = FALSE );
	long			LastSelectedRow( BOOL bInverse = FALSE );
	long			PrevSelectedRow();
	long			NextSelectedRow();

	
	Window& 		GetDataWindow() const
					{ return *pDataWin; }
	Rectangle		GetRowRectPixel( long nRow,
							BOOL bRelToBrowser = TRUE ) const;
	Rectangle		GetFieldRectPixel( long nRow, USHORT nColId,
							BOOL bRelToBrowser = TRUE  ) const;
    BOOL            IsFieldVisible( long nRow, USHORT nColId,
                                    BOOL bComplete = FALSE ) const;
	long			GetRowAtYPosPixel( short nY,
							BOOL bRelToBrowser = TRUE  ) const;
	USHORT			GetColumnAtXPosPixel( short nX,
							BOOL bRelToBrowser = TRUE  ) const;

	void			Clear();
    void            RowRemoved( long nRow );
    void            RowModified( long nRow, USHORT nColId = USHRT_MAX );
    void            RowInserted( long nRow );

	
	void			ReserveControlArea( USHORT nWidth = USHRT_MAX );
	Rectangle		GetControlArea() const;
	void			Dispatch( USHORT nId );
	void			SetMode( BrowserMode nMode = 0 );
};


#endif
